Module operation

Module operation 

Modules

builder
This module provides functionality related to building user defined operations.
builtin
marker
Markers are tags that can be applied to nodes in the concrete graph. They do not exist abstractly. Markers are primarily used to hide nodes from shape queries.
query
signature
user_defined

Structs

ConcreteData
Additional, global data that can be used and modified by operations.
OperationContext
Contains available operations

Enums

Operation
OperationError
SubstitutionError

Traits

BuiltinOperation

Functions

get_substitution
Returns the pattern subst to input graph node key mapping, if the operation is applicable.
run_from_concrete
run_operation

Type Aliases

OperationId
OperationResult