pub struct AbstractOutputChanges<S: Semantics> {
pub new_nodes: HashMap<AbstractOutputNodeMarker, S::NodeAbstract>,
pub new_edges: HashMap<AbstractSignatureEdgeId, S::EdgeAbstract>,
pub maybe_changed_nodes: HashMap<SubstMarker, S::NodeAbstract>,
pub maybe_changed_edges: HashMap<ParameterEdgeId, S::EdgeAbstract>,
pub maybe_deleted_nodes: HashSet<SubstMarker>,
pub maybe_deleted_edges: HashSet<ParameterEdgeId>,
}Expand description
The changes to the graph that an operation will cause.
Fields
new_nodes: HashMap<AbstractOutputNodeMarker, S::NodeAbstract>New nodes that are guaranteed to be created with a value of the given type.
new_edges: HashMap<AbstractSignatureEdgeId, S::EdgeAbstract>New edges that are guaranteed to be created with a value of the given type.
maybe_changed_nodes: HashMap<SubstMarker, S::NodeAbstract>Pre-existing nodes that may have been modified to be of the given type.
maybe_changed_edges: HashMap<ParameterEdgeId, S::EdgeAbstract>Pre-existing edges that may have been modified to be of the given type.
maybe_deleted_nodes: HashSet<SubstMarker>Pre-existing nodes that may have been deleted by the operation.
maybe_deleted_edges: HashSet<ParameterEdgeId>Pre-existing edges that may have been deleted by the operation.
Implementations
Sourceimpl<S: Semantics> AbstractOutputChanges<S>
impl<S: Semantics> AbstractOutputChanges<S>
pub fn new() -> Self
Sourcepub fn is_subtype_of(&self, other: &Self) -> bool
pub fn is_subtype_of(&self, other: &Self) -> bool
Returns true if self can be used wherever other is expected, i.e., self <: other.
pub fn apply_abstract( &self, g: &mut GraphWithSubstitution<'_, AbstractGraph<S>>, ) -> AbstractOperationOutput<S>