Modules
- builder
- This module provides functionality related to building user defined operations.
- builtin
- marker
- Markers are tags that can be applied to nodes in the concrete graph. They do not exist abstractly. Markers are primarily used to hide nodes from shape queries.
- query
- signature
- user_
defined
Structs
- Concrete
Data - Additional, global data that can be used and modified by operations.
- Operation
Context - Contains available operations
Enums
Traits
Functions
- get_
substitution - Returns the pattern subst to input graph node key mapping, if the operation is applicable.
- run_
from_ concrete - run_
operation