pub enum BuilderInstruction<S: Semantics> {
Show 22 variants
ExpectParameterNode(SubstMarker, S::NodeAbstract),
ExpectContextNode(SubstMarker, S::NodeAbstract),
ExpectParameterEdge(SubstMarker, SubstMarker, S::EdgeAbstract),
StartQuery(S::BuiltinQuery, Vec<AbstractNodeId>),
EnterTrueBranch,
EnterFalseBranch,
StartShapeQuery(AbstractOperationResultMarker),
EndQuery,
ExpectShapeNode(AbstractOutputNodeMarker, S::NodeAbstract),
ExpectShapeNodeChange(AbstractNodeId, S::NodeAbstract),
ExpectShapeEdge(AbstractNodeId, AbstractNodeId, S::EdgeAbstract),
SkipMarker(Marker),
SkipAllMarkers,
AddNamedOperation(AbstractOperationResultMarker, BuilderOpLike<S>, Vec<AbstractNodeId>),
AddBangOperation(NamedMarker, BuilderOpLike<S>, Vec<AbstractNodeId>),
AddOperation(BuilderOpLike<S>, Vec<AbstractNodeId>),
ReturnNode(AbstractNodeId, AbstractOutputNodeMarker, S::NodeAbstract),
ReturnEdge(AbstractNodeId, AbstractNodeId, S::EdgeAbstract),
RenameNode(AbstractNodeId, NamedMarker),
Finalize,
SelfReturnNode(AbstractOutputNodeMarker, S::NodeAbstract),
Diverge(String),
}Variants
ExpectParameterNode(SubstMarker, S::NodeAbstract)
ExpectContextNode(SubstMarker, S::NodeAbstract)
ExpectParameterEdge(SubstMarker, SubstMarker, S::EdgeAbstract)
StartQuery(S::BuiltinQuery, Vec<AbstractNodeId>)
EnterTrueBranch
EnterFalseBranch
StartShapeQuery(AbstractOperationResultMarker)
EndQuery
ExpectShapeNode(AbstractOutputNodeMarker, S::NodeAbstract)
ExpectShapeNodeChange(AbstractNodeId, S::NodeAbstract)
ExpectShapeEdge(AbstractNodeId, AbstractNodeId, S::EdgeAbstract)
SkipMarker(Marker)
SkipAllMarkers
AddNamedOperation(AbstractOperationResultMarker, BuilderOpLike<S>, Vec<AbstractNodeId>)
AddBangOperation(NamedMarker, BuilderOpLike<S>, Vec<AbstractNodeId>)
AddOperation(BuilderOpLike<S>, Vec<AbstractNodeId>)
ReturnNode(AbstractNodeId, AbstractOutputNodeMarker, S::NodeAbstract)
ReturnEdge(AbstractNodeId, AbstractNodeId, S::EdgeAbstract)
RenameNode(AbstractNodeId, NamedMarker)
Rename a dynamic output marker. Invariants in the interpreter require that this is never a parameter node. (E.g., since we may want to return it)
Finalize
SelfReturnNode(AbstractOutputNodeMarker, S::NodeAbstract)
Asserts that the current operation will return a node with the given abstract value and name.
Diverge(String)
Diverge with a crash message. Has a static effect: The branch is considered to never return, hence merges will always take the other branch.